/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl;

import java.util.Map;
import java.util.TreeMap;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Compiler;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * This compiler call sequentially each client compiler according to the
 * alphabetical order.
 */
public class CompilerDispatcher implements Compiler, BindingController {
  /**
   * Name of the collection interface bound to the
   * {@link org.objectweb.fractal.adl.components.PrimitiveCompiler
   * PrimitiveCompilers} used by this compiler.
   */
  public static final String   COMPILERS_BINDING = "client-compiler";

  /** The primitive compilers used by this compiler. */
  public Map<String, Compiler> compilersItf      = new TreeMap<String, Compiler>();

  // --------------------------------------------------------------------------
  // Implementation of the Compiler interface
  // --------------------------------------------------------------------------

  public void compile(final Definition definition, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    for (final Compiler compiler : compilersItf.values())
      compiler.compile(definition, tasks, context);
  }

  // --------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // --------------------------------------------------------------------------

  public String[] listFc() {
    return compilersItf.keySet().toArray(new String[compilersItf.size()]);
  }

  public Object lookupFc(final String itf) {
    if (itf.startsWith(COMPILERS_BINDING)) {
      return compilersItf.get(itf);
    }
    return null;
  }

  public void bindFc(final String itf, final Object value) {
    if (itf.startsWith(COMPILERS_BINDING)) {
      compilersItf.put(itf, (Compiler) value);
    }
  }

  public void unbindFc(final String itf) {
    if (itf.startsWith(COMPILERS_BINDING)) {
      compilersItf.remove(itf);
    }
  }

}
