/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.compiler;

import org.objectweb.fractal.cecilia.adl.plugin.PluginUserAttributes;

/**
 * {@link org.objectweb.fractal.api.control.AttributeController} interface for
 * {@link OutputCompilerSelector} component. It extends the
 * {@link PluginUserAttributes} and adds a <code>defaultFormat</code>
 * attribute.
 */
public interface OutputCompilerSelectorAttributes extends PluginUserAttributes {

  /**
   * Sets the default output format to be used by {@link OutputCompilerSelector}
   * if a component does not specify one.
   * 
   * @param defaultFormat the new value of the <code>defaultFormat</code>.
   */
  void setDefaultFormat(String defaultFormat);

  /**
   * Returns the value of the <code>defaultFormat</code>.
   * 
   * @return the value of the <code>defaultFormat</code>.
   * @see #setDefaultFormat(String)
   */
  String getDefaultFormat();

  /**
   * Sets the default output format for the top level component.
   * 
   * @param defaultTopLevelFormat the new value of the
   *          <code>defaultTopLevelFormat</code>.
   * @see #setDefaultFormat(String)
   */
  void setDefaultTopLevelFormat(String defaultTopLevelFormat);

  /**
   * Returns the value of the <code>defaultTopLevelFormat</code>.
   * 
   * @return the value of the <code>defaultTopLevelFormat</code>.
   * @see #setDefaultTopLevelFormat(String)
   */
  String getDefaultTopLevelFormat();

}
