/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.idl;

import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.cecilia.adl.types.TypeDecorationUtil;
import org.objectweb.fractal.task.core.Task;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * IDL compiler for component interfaces.
 */
public class IDLCompilerImpl extends AbstractIDLCompiler {

  // --------------------------------------------------------------------------
  // Implementation of the IDLCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final IDLDefinition idlDefinition, final ComponentContainer container,
      final TaskMap tasks, final Map<Object, Object> context)
      throws ADLException {

    String taskName = "IDL";
    if (taskSuffix != null) {
      taskName += '-' + taskSuffix;
    }
    try {
      tasks.getTask(taskName, idlDefinition);
      return;
    } catch (final NoSuchElementException e) {
      final IDLDefinitionTask itfTask = new IDLDefinitionTask(idlDefinition,
          buildersItf);
      final TaskMap.TaskHole itfTaskHole = tasks.addTask(taskName,
          idlDefinition, itfTask);
      final TaskMap.TaskHole compilerTaskHole = tasks.getTaskHole(
          "source-file", TypeDecorationUtil.getTypeDecoration(container));
      compilerTaskHole
          .addDependency(itfTaskHole, Task.PREVIOUS_TASK_ROLE, null);
    }
  }
}
