/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.implementations;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.adl.implementations.ImplementationContainer;
import org.objectweb.fractal.cecilia.adl.components.AbstractPrimitiveCompilerPluginUser;
import org.objectweb.fractal.cecilia.adl.plugin.PluginException;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * Delegates compilation to a client {@link PrimitiveCompiler} depending on the
 * implementation language of each component.
 */
public class LanguageCompilerSelector
    extends
      AbstractPrimitiveCompilerPluginUser implements PrimitiveCompiler {

  // --------------------------------------------------------------------------
  // Implementation of the PrimitiveCompiler interface
  // --------------------------------------------------------------------------

  public void compile(final List<ComponentContainer> path,
      final ComponentContainer container, final TaskMap tasks,
      final Map<Object, Object> context) throws ADLException {
    String implementationLanguage = null;
    if (container instanceof ImplementationContainer) {
      ExtendedImplementation implem = null;
      if (((ImplementationContainer) container).getImplementation() instanceof ExtendedImplementation)
        implem = (ExtendedImplementation) ((ImplementationContainer) container)
            .getImplementation();
      if (implem != null) {
        implementationLanguage = implem.getLanguage();
      }

      if (implementationLanguage == null)
        throw new ADLException(
            "Can't find the implementation language of the component",
            (Node) implem, null);
    } else
      throw new ADLException("Can't find the implementation of the component",
          (Node) container, null);

    try {
      getPrimitiveCompilerPlugin(implementationLanguage, context).compile(path,
          container, tasks, context);
    } catch (final PluginException e) {
      throw new ADLException("Can't find plugin for implementation language '"
          + implementationLanguage + "'", (Node) container, e);
    }
  }
}
