/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006 FranceTelecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Juraj Polakovic
 */
// TODO remove support for ThinkV2 ADL.

package org.objectweb.fractal.cecilia.adl.parser;

import java.io.InputStream;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.ADLNotFoundException;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Loader;
import org.objectweb.fractal.adl.xml.XMLNodeFactory;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.cecilia.adl.parser.v2.ADLParser;
import org.objectweb.fractal.cecilia.adl.parser.v2.ParseException;
import org.xml.sax.SAXException;

/**
 * @author juraj
 */
public class PlainADLLoader implements Loader, BindingController {

  /** node factory client interface name. */
  public static final String NODE_FACTORY_ITF = "node-factory";

  /** {@link XMLNodeFactory} client interface. */
  public XMLNodeFactory      nodeFactoryItf;

  // --------------------------------------------------------------------------
  // Implementation of the Loader interface
  // --------------------------------------------------------------------------

  public Definition load(final String name, final Map<Object, Object> context)
      throws ADLException {
    // System.err.println("Loading plain ADL: " + name);

    // get the name of the corresponding file
    final String file = name.replace('.', '/') + ".adl";

    // get input stream on it using class loader
    ClassLoader cl = null;
    if (context != null) {
      cl = (ClassLoader) context.get("classloader");
    }
    if (cl == null) {
      cl = getClass().getClassLoader();
    }
    final InputStream is = cl.getResourceAsStream(file);
    if (is == null) {
      throw new ADLNotFoundException("Cannot find '" + file
          + "' in the classpath");
    }

    // create a new parser using the stream
    final ADLParser parser = new ADLParser(is);

    try {
      // parse the stream and construct the corresponging AST
      return parser.CompilationUnit(nodeFactoryItf, file);
    } catch (final ParseException e) {
      throw new ADLException("Cannot load '" + name + "'", null, e);
    } catch (final SAXException e) {
      throw new ADLException("Cannot load '" + name + "'", null, e);
    }
  }

  public void bindFc(final String itfName, final Object value)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface named can't be null");
    }

    if (itfName.equals(NODE_FACTORY_ITF)) {
      try {
        this.nodeFactoryItf = (XMLNodeFactory) value;
      } catch (final RuntimeException e) {
        e.printStackTrace();
        throw new IllegalBindingException(
            "Unexpected type for client interface '" + itfName + "'");
      }
    } else {
      throw new NoSuchInterfaceException("There is no interface named '"
          + NODE_FACTORY_ITF + "'");
    }

  }

  public String[] listFc() {
    return new String[]{NODE_FACTORY_ITF};
  }

  public Object lookupFc(final String itfName) throws NoSuchInterfaceException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface named can't be null");
    }

    if (itfName.equals(NODE_FACTORY_ITF)) {
      return this.nodeFactoryItf;
    } else {
      throw new NoSuchInterfaceException("No interface named '" + itfName + "'");
    }
  }

  public void unbindFc(final String itfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {

    if (itfName == null) {
      throw new IllegalArgumentException("Interface named can't be null");
    }

    if (itfName.equals(NODE_FACTORY_ITF)) {
      this.nodeFactoryItf = null;
    }
  }
}
