/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.composite.c.controllers;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.cecilia.adl.bindings.BindingBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * @author ozcane
 */
public class BCBootCompositeInstanceBuilder implements BindingBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the BindingBuilder interface
  // --------------------------------------------------------------------------

  public Object bindComponent(final Object componentType,
      final Object componentInstance, final int bindingType,
      final Object client, final String clientItf, final Object server,
      final String serverItf, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    if (client == null) {
      final CodeWriter cw = new CodeWriter(
          "Composite Binding Controller Initialization Builder");
      final String instanceName = componentInstance.toString()
          .replace('.', '_');
      // bindings are ignored since binding between first stage and second stage
      // is defined as a static reference.
      cw.append("struct BindingDesc_t ").append(instanceName).append(
          "_bindings[] ={};").endl();
      for (final Object codePiece : codePieces.values()) {
        cw.appendln(codePiece.toString());
      }

      return cw.toString();
    }

    final CodeWriter cw = new CodeWriter();
    final String serverS = server.toString().replace('.', '_');

    // In the case of normal bindings, this controller defines global variables
    if ((bindingType & BindingBuilder.NEI_BINDING_MASK) == NORMAL_BINDING) {
      if (!clientItf.equals("boot"))
        throw new ADLException(
            "A boot controller can only accept an internal binding for a client"
                + " interface called 'boot'", (Node) null);
      cw.append("#include \"").append(server.toString().replace('.', '/'))
          .append(".adl.h\"").endl();
      cw.append("void *__").append(clientItf).append("_itf__ = &(").append(
          serverS).append(".").append(serverItf).append(");").endl();
      return cw.toString();
    }

    // XXX why do not return null ?
    return "";
  }
}
