/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.composite.c.controllers;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ContentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the declaration of the content controller structure of a composite
 * component.
 */
public class CCInstanceBuilder implements ContentBuilder {

  public Object addContent(final Object componentType,
      final Object componentInstance, final Object[] subComponentTypes,
      final Object[] subComponentInstances,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter("Composite Builder");
    final String instanceName = componentInstance.toString().replace('.', '_');

    final String[] subCompNames = new String[subComponentInstances.length];
    for (int i = 0; i < subComponentInstances.length; i++) {
      subCompNames[i] = subComponentInstances[i].toString().replace('.', '_');
    }

    cw.append("/* extern reference for the exported interface tables of ")
        .append(instanceName).append("'s sub components*/").endl();
    for (int i = 0; i < subComponentInstances.length; i++) {
      cw.append("#include \"").append(
          subComponentTypes[i].toString().replace('.', '/')).append(".adl.h\"")
          .endl();
      cw.append("extern struct ").append(
          subComponentTypes[i].toString().replace('.', '_')).append(
          "_exporteds ").append(subCompNames[i]).append(";").endl();
    }

    cw.append("/* Defining the ").append(instanceName).append(
        "'s sub component index numbers*/").endl();
    for (int i = 0; i < subComponentInstances.length; i++) {
      cw.append("#define ").append(subCompNames[i]).append("_exp ").append(i)
          .endl();
    }
    cw.endl();

    cw.append("/* sub component table for the composite ").append(instanceName)
        .append(" */").endl();
    cw.append("struct SubComponentDesc_t ").append(instanceName).append(
        "_subComponents[] = {").endl();
    for (int i = 0; i < subComponentInstances.length; i++) {
      // This part is probably not correct because it does not handle
      // composite sub components
      cw.append("{&(").append(
          subComponentInstances[i].toString().replace('.', '_')).append(
          ".component), // instance").endl();
      cw.append("SUB_COMPONENT_NOT_INITIALIZED, // state").endl();
      cw.appendln("#ifdef DEBUGCOMPOSITE");
      cw.append("\"").append(subCompNames[i]).append("\", // name").endl();
      cw.append("\"").append(subComponentTypes[i].toString().replace('.', '_'))
          .append("\", // implementationName").endl();
      cw.appendln("#endif // DEBUGCOMPOSITE");
      cw.appendln("},");
    }
    cw.appendln("};");

    return cw.toString();
  }

}
