/***
 * Cecilia ADL Compiler
 * Copyright (C) 2005 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.components;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ComponentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the implementation of <code>factory</code> interface for a ThinkMC
 * primitive component.
 */
public class FactoryImplementationBuilder implements ComponentBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the ImplementationBuilder interface
  // --------------------------------------------------------------------------

  public Object createComponent(final Object componentType,
      final Object componentInstance, final Object controllerDesc,
      final Object contentDesc, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    final CodeWriter cw = new CodeWriter("Factory Implementation Builder");
    final String componentCName = componentType.toString().replace('.', '_');

    cw.append("#ifdef KORTEX_LIBC").endl();
    cw.append("#include <libc/string.h>").endl();
    cw.append("#else").endl();
    cw.append("#include <string.h>").endl();
    cw.append("#endif // KORTEX_LIBC").endl();
    // instantiate method implementation ---------------------------------------
    cw.append("int ").append("METHOD(factory, newFcInstance)(void * _this, ")
        .append("Rfractal_api_Component ** instance) {").endl();
    cw.append("struct ").append(componentCName).append("_t *originalComp = ")
        .append("(struct ").append(componentCName)
        .append("_t *) GET_MY_OWNER;").endl();
    cw.append("Rmemory_api_Allocator *alloc_itf = REQUIRED.factory_allocator;")
        .endl();
    cw.append("struct ").append(componentCName).append("_t *newComp = (")
        .append("struct ").append(componentCName).append(
            "_t *) CALL(alloc_itf, alloc, sizeof(").append("struct ").append(
            componentCName).append("_t));").endl();
    cw.append((String) codePieces.get("controller.instantiate.vardecl")).endl();
    cw.append((String) codePieces.get("interfaces.instantiate.vardecl")).endl();

    cw.append("memcpy(newComp, originalComp, sizeof(").append("struct ")
        .append(componentCName).append("_t));").endl();

    cw.append((String) codePieces.get("controller.instantiate.implem")).endl();
    cw.append((String) codePieces.get("interfaces.instantiate.implem")).endl();

    cw.append("*instance = newComp;");
    cw.append("return fractal_api_ErrorConst_OK;").endl();

    cw.append("}").endl().endl();

    // destroy method implementation -------------------------------------------
    cw.append("int METHOD(factory, destroyFcInstance)").append(
        "(void * _this, Rfractal_api_Component *comp) {").endl();
    cw.append("struct ").append(componentCName).append("_t *originalComp = (")
        .append("struct ").append(componentCName).append("_t *) comp;").endl();
    cw.append("Rmemory_api_Allocator *alloc_itf = REQUIRED.factory_allocator;")
        .endl();

    cw.append((String) codePieces.get("controller.destroy.vardecl")).endl();
    cw.append((String) codePieces.get("interfaces.destroy.vardecl")).endl();
    cw.append((String) codePieces.get("controller.destroy.implem")).endl();
    cw.append((String) codePieces.get("interfaces.destroy.implem")).endl();

    cw.append("CALL(alloc_itf, free, (jbyte *) originalComp);").endl();
    cw.append("return fractal_api_ErrorConst_OK;").endl();

    cw.append("}").endl().endl();

    return cw.toString();
  }

}
