/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.Map;

import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.cecilia.adl.attributes.AttributeBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Build the instantiation if the internal data of the attribute controller.
 */
public class ACInstanceBuilder implements AttributeBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the AttributesBuilder interface
  // --------------------------------------------------------------------------

  public Object setAttributes(final Object componentType,
      final Object componentInstance, final Object attributeAST,
      final Attribute[] fields, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter(
        "Attribute Controller Instantiation Builder");
    final String instanceName = componentInstance.toString().replace('.', '_');
    cw.append("struct ").append(componentType.toString().replace('.', '_'))
        .append("_attributedskeleton ").append(instanceName).append(
            "_attributes = {").append(fields.length).append(", {").endl();
    for (final Attribute element : fields)
      cw.append("{\"").append(element.getName()).append("\", &").append(
          instanceName).append(".attributes.").append(element.getName())
          .append("},").endl();
    cw.append("}};");
    return cw.toString();
  }
}
